/*

Header file for send UDP spoofed packets (FULL version)
by Luigi Auriemma
e-mail: aluigi@autistici.org
web:    aluigi.org

VERSION 0.1.2


For use this header file, simply add #include "send_udp_full.h" to
your source code.


This function need the following parameters:

- src     = source hostname or IP (example: "www.pivx.com")
- dst     = destination hostname or IP
- sport   = source port (NO htons) (example: port 80 = 80)
- dport   = destination port (NO htons)
- data    = data to insert in the packet
- datalen = size of the data to insert in the packet
- num     = how many packets send

NOTE:
- if one of the port is equal to 0, it will be randomized
- ID and TTL of the IP header are randomized but are the same for all
  the packets
- if you need a fastest and tiny function for send spoofed UDP
  datagrams, you must use send_udp_fast.h


Example:
send_udp_full(
    "www.riaa.org",        //source
    "www.microsoft.com",    //destination
    0,            //random source port
    135,            //destination port
    "Hello how are you?",    //data (NULL for empty packet)
    18,            //data length (0 for empty packet)
    3);            //number of packets


LINUX VERSION

*/

#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netdb.h>
#include <string.h>
#include <time.h>




#define IPSZ        sizeof(struct iphdr)
#define UDPSZ        sizeof(struct udphdr)
#define PSEUDOSZ    sizeof(struct pseudohdr)
#define SIZE        (IPSZ + UDPSZ + datalen)
#define SENDUDPDELAY        0




void send_udp_full(
    u_char    *src,
    u_char    *dst,
    u_short    sport,
    u_short    dport,
    u_char    *data,
    u_short    datalen,
    u_int    num);
u_short in_cksum(u_short *addr, int len);
u_int resolv(char *host);
void std_err(void);



struct pseudohdr {
    u_int32_t    saddr;
    u_int32_t    daddr;
    u_int8_t    zero;
    u_int8_t    protocol;
    u_int16_t    length;
} *pseudohdr;


/*
for more speed the ports must be given with htons (big endian) so the
function doesn't need to make this operation everytime

it supports the randomization of the source and dest port if equal to 0
*/


void send_udp_full(
    u_char    *src,
    u_char    *dst,
    u_short    sport,
    u_short    dport,
    u_char    *data,
    u_short    datalen,
    u_int    num) {

    u_int    ipsrc,
        ipdst;
    u_char    buff[SIZE],
        pseudobuff[PSEUDOSZ + UDPSZ + datalen],
        *datapck;
    struct    sockaddr_in     peer;
    struct    iphdr    *iphdr;
    struct    udphdr    *udphdr;
    int    sd,
        err;


    srand(time(NULL));

    if(!sport) sport = rand();
    if(!dport) dport = rand();

    ipsrc                = resolv(src);
    ipdst                = resolv(dst);
    peer.sin_addr.s_addr = ipdst;
    peer.sin_port        = htons(dport);
    peer.sin_family      = AF_INET;

    iphdr     = (struct iphdr *)buff;
    udphdr    = (struct udphdr *)(buff + IPSZ);
    datapck   = (u_char *)(buff + IPSZ + UDPSZ);
    pseudohdr = (struct pseudohdr *)pseudobuff;


    /* build IP header */
    iphdr->ihl      = 5;
    iphdr->version  = 4;
    iphdr->tos      = 0x10;
    iphdr->tot_len  = SIZE;
    iphdr->id       = rand();
    iphdr->frag_off = 0;
    iphdr->ttl      = (rand() >> 24) + 120;
    iphdr->protocol = IPPROTO_UDP;
    iphdr->check    = 0;
    iphdr->saddr    = ipsrc;
    iphdr->daddr    = ipdst;

    /* build UDP header */
    udphdr->source = htons(sport);
    udphdr->dest   = htons(dport);
    udphdr->check  = 0;
    udphdr->len    = htons(UDPSZ + datalen);

    /* build data */
    memcpy(datapck, data, datalen);

    /* build pseudo header for calculate checksum (copy UDP header and data in it) */
    memcpy(pseudobuff + PSEUDOSZ, udphdr, UDPSZ + datalen);

    pseudohdr->saddr    = iphdr->saddr;
    pseudohdr->daddr    = iphdr->daddr;
    pseudohdr->zero     = 0;
    pseudohdr->protocol = IPPROTO_UDP;
    pseudohdr->length   = udphdr->len;

    udphdr->check = in_cksum((u_short *)pseudobuff, PSEUDOSZ + UDPSZ + datalen);


    /* send all */
    sd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
    if(sd < 0) std_err();

    for(; num > 0; num--) {
        err = sendto(sd, buff, SIZE, 0, (struct sockaddr *)&peer, sizeof(peer));
        if(err < 0) std_err();
        usleep(SENDUDPDELAY);
    }
    close(sd);
}




u_short in_cksum(u_short *addr, int len) {
        int        sum = 0;
        u_short        answer = 0;
        register    u_short *w = addr;
        register int    nleft = len;

        while(nleft > 1)  {
                sum += *w++;
                nleft -= 2;
        }
        if(nleft == 1) {
                *(u_char *)(&answer) = *(u_char *)w ;
                sum += answer;
        }
        sum = (sum >> 16) + (sum & 0xffff);
        sum += (sum >> 16);
        answer = ~sum;
        return(answer);
}




u_int resolv(char *host) {
        struct    hostent *hp;
        u_int    host_ip;

        host_ip = inet_addr(host);
        if(host_ip == INADDR_NONE) {
            hp = gethostbyname(host);
            if(!hp) {
                printf("\nError: Unable to resolve hostname (%s)\n", host);
                exit(1);
            } else host_ip = *(u_int *)(hp->h_addr);
        }

        return(host_ip);
}



void std_err(void) {
        perror("\nError");
        exit(1);
}

